# Base Content Class for NeoBoard
# BCC adds structured text/caching support. Whenever the raw content changes, it re-renders it. 

from Globals import InitializeClass
from OFS.DTMLDocument import DTMLDocument
from StructuredText import HTMLClass, DocumentClass, ST

Document = DocumentClass.DocumentClass()
HTMLNG = HTMLClass.HTMLClass()

class NB_BaseContent(DTMLDocument):
	"""
	NeoBoard Base Content Class
	"""
	
	meta_type='NeoBoard Base Content'

	def getRawContent(self):
		if not hasattr(self.aq_base, 'raw'):
			return ''
	
		return self.raw
		
	def getRenderedContent(self, REQUEST=None):
		if not hasattr(self.aq_base, '_rendered_content'):
			if not self.updateContent():
				return ''
		
		return self._rendered_content

	def updateContent(self):
	
		if not hasattr(self.aq_base, 'raw'):
			return 0
		
		self._renderContent()
		return 1

			
	def _renderContent(self):	
		st = ST.Basic(self.getRawContent())
		doc = Document(st)
		self._rendered_content = HTMLNG(doc, header=0)
		
	def selfUpgrade(self):		
		pass		
	
				
InitializeClass(NB_BaseContent)