# Utility methods for NeoBoard 

from Globals import DTMLFile
from string import replace, join, split, count
import re
from OFS.Image import Image
import os, os.path, string, types, urllib

def getDTML(name):
	return DTMLFile(name, globals())

def addDTML(obj, id, title, fname):
	f = open(fname)     
	file = f.read()
	f.close() 
	obj.manage_addDTMLMethod(id,title,file)
	return getattr(obj,id)

def addImage(obj, id, title, fname):  
	f = open(fname,'rb')     
	
	img = f.read()
	f.close()     
	tlen = len(img)     
	new_id = obj.manage_addImage(id,img,title)

def natSort(list):
	""" Natural-sort page_1, page_11, page_2... list."""
	
	list.sort(lambda x,y: cmp(int(x.split('_')[1]), int(y.split('_')[1])))
	
	return list		
	
def getNextElementName(list, prefix='page_'):
	""" natSort the list and pop it to get the next probable name for the given element.
	    Ex: page_2 -> page_3
	        c_1 -> c_2
	"""
	if not len(list):
		return prefix + '1'
	else:
		natSort(list)
		return prefix + str(int(list.pop().split('_')[1]) + 1)

def setFormMsg(req, msg):
	req.set('FORM_MESSAGE', msg)
	return req
		
def setFormError(req, key, msg):
	req.set('FORM_ERROR', 1)
	req.set('FORM_ERROR_' + key, msg)	
	return req
	
def getReqAttr(req, attr):
	if req.has_key(attr):
		if req[attr] == '':
			return None
		else:
			return req[attr]
	else:
		return None	

		
def traverseFolder(folder, meta_type=None, skey=None, reverse=0, default=None, default_reverse=0):
	pool=[]
	
	if meta_type is None:
		objects = folder.objectValues()
	else:
		objects = folder.objectValues(meta_type)

	if default:
		objects.sort(lambda x,y, default=default: cmp(x[default], y[default]))
		if default_reverse:
			objects.reverse()
	
	for obj in objects:
		pool.append(obj)
		pool = pool + traverseFolder(obj, meta_type, skey, reverse)
		
	if skey:
		pool.sort(lambda x,y, skey=skey: cmp(x[skey], y[skey]))
		if reverse:
			pool.reverse()
	
	return pool


def cutString(str, chunk=10, ellipsis='...'):
	if len(str) <= chunk or not chunk:
		return str
		
	match = re.compile('\s*|\w*').match(str, chunk)
	if match is None:

		ret_size = chunk
	
		return str[:ret_size] + ellipsis
	else:
		ret_size = match.start() - 1
		return str[:ret_size] + ellipsis

def isEmailValid(email, bad_domains=''):
	if type(bad_domains) == type([]) and len(bad_domains) > 0:
		for dom in bad_domains:
			if re.compile('@'+dom).search(email) is not None:
				return 0

	if re.compile('\s').search(email) is not None:
		return 0
	
	if re.compile(r'^[_\-\.0-9a-z]+@([0-9a-z][_0-9a-z\.]+)\.([a-z]{2,4}$)', re.IGNORECASE).search(email) is None:
		return 0
	
	return 1

def copyFile(infile, outfile):
	if type(infile) is types.StringType: 
		instream = open(infile, 'rb')
		close_in = 1
	else:
		instream = infile
		close_in = 0

	if type(outfile) is types.StringType: 
		outstream = open(outfile, 'wb')
		close_out = 1
	else:
		outstream = outfile
		close_out = 0

	blocksize = 2<<16
	block = instream.read(blocksize)
	outstream.write(block)
	while len(block)==blocksize:
		block = instream.read(blocksize)
		outstream.write(block)
	try: 
		instream.seek(0)
	except: 
		pass
	
	if close_in: 
		instream.close()
	if close_out: 
		outstream.close()