# NeoBoard Article Class

from Globals import InitializeClass, MessageDialog
from AccessControl import ClassSecurityInfo
from NB_Utils import getDTML, getNextElementName, getReqAttr, setFormMsg, setFormError, cutString, copyFile, isEmailValid
from DateTime.DateTime import DateTime
from string import replace, join, split, count
from NB_BaseFolder import NB_BaseFolder
import os, mimetypes

addNeoBoardArticleForm=getDTML('dtml/NeoBoardArticleAdd')
def addNeoBoardArticle(self, 
		title='', 
		data='',
		author='',
		author_email=None,
		password=None,
		category='',
		attachment='',
		attachment_title='',
		attachment_description='',
		mail_reply=0,
		is_notice=0,
		REQUEST=None):
	""" Add NeoBoard Article"""

	if self.isReadOnly():
		if not self.isNeoBoardManager():
			return self.addNeoBoardArticleForm(REQUEST=setFormError(REQUEST, 'title', "Read-only board. You can't post an article."))		

	if not author:
		return self.addNeoBoardArticleForm(setFormError(REQUEST, 'author', 'Please enter your username!'))

	if author_email and not isEmailValid(author_email):
		return self.addNeoBoardArticleForm(setFormError(REQUEST, 'author_email', 'Your email address is invalid!'))
	
	if not title:
		return self.addNeoBoardArticleForm(setFormError(REQUEST, 'title', 'Please enter a title!'))

	if not data:
		return self.addNeoBoardArticleForm(setFormError(REQUEST, 'data', 'Empty article is not allowed!'))
	
	
	if is_notice:
		instance = NeoBoardNotice()
		id = 'n_' + str(self.getNextArticleNum(1))

	else:
		instance = NeoBoardArticle()
		id = 'a_' + str(self.getNextArticleNum())

	instance.id = id	
	instance.title=str(title)
	
	if title:
		instance.menu_title=instance.title
	else:
		instance.menu_title=instance.id	
			
	instance.munge(data)
	instance.category=category
	instance.mail_reply=mail_reply
	instance.setReadCount(0)
	instance.setNextReplyNum(0)
	instance.setTotalNumReplies(0)
	instance.setTotalNumVotes(0)
	

	instance.author=author
	instance.author_email=author_email
		
	instance.creation_date = DateTime()	
	instance.password = str(password)

	if attachment.filename:
		if not self.checkSize(attachment):
			return self.addNeoBoardArticleForm(REQUEST=setFormError(REQUEST, 'attachment', 'Maximum upload file size exceeded!'))		
		
	self._setObject(id, instance)
	
	if attachment.filename:			
		self._getOb(id).uploadFile(attachment, attachment_title, attachment_description)
	else:
		self.attachment_file_id=''		
	
	self.setTotalNumArticles(self.getTotalNumArticles()+1)
	
	if self.isMailEnabled():
		self._getOb(id).mailArticle()

	self._getOb(id)._setRemoteIP(REQUEST)		

	if REQUEST is not None:
    		return REQUEST.RESPONSE.redirect(self.absolute_url() + self._getOb(id)._getPageQueryString(REQUEST))		

class NeoBoardArticle(NB_BaseFolder):
	"""
	NeoBoard Article Class
	"""
	meta_type='NeoBoard Article'
	icon='misc_/NeoBoard/neoboard_article_icon'     
	
	_max_thread_depth = 5
	
	manage_editForm = getDTML('dtml/NeoBoardArticleEdit')
	
	manage_options=({'label':'Contents', 'icon':'', 'action':'manage_main', 'target':'manage_main'},
			{'label':'View', 'icon':'', 'action':'index_html', 'target':'manage_main'},                 
			{'label':'Properties', 'action':'manage_editForm'},
			{'label':'Security', 'icon':'', 'action':'manage_access', 'target':'manage_main'},
			{'label':'Undo', 'icon':'', 'action':'manage_UndoForm', 'target':'manage_main'},
	)  

	security = ClassSecurityInfo()	
	security.declareObjectProtected('View')

	article_template = getDTML('dtml/NeoBoardArticle')
	reply =getDTML('dtml/NeoBoardArticleReplyAdd')
	modify = getDTML('dtml/NeoBoardArticleModify')	
	
	def __init__(self):
		pass

	def index_html(self, REQUEST={}, RESPONSE=None):
		"""
			Own index_html
		"""
		REQUEST.set('CURRENT_ARTICLE_URL', self.absolute_url())
		header = apply(self.neoboard_header.__call__, (self, REQUEST, RESPONSE))
	        footer = apply(self.neoboard_footer.__call__, (self, REQUEST, RESPONSE))
		self.readCountUp()
		
      		return header + self.article_template(REQUEST) + footer

	def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):      			
		return self.article_template()		
		
	def vote(self, REQUEST, RESPONSE):
		"""
			Vote for this article
		"""		
		
		self.setTotalNumVotes(self.getTotalNumVotes()+1)
		return self.index_html(REQUEST, RESPONSE)

	def getReadCount(self):
		return self.read_count

	def setReadCount(self, count=0):
		self.read_count = count
	
	def readCountUp(self):
		self.setReadCount(self.getReadCount() + 1)
		
	def getAuthor(self):
		return self.author

	def getAuthorEmail(self):
		return self.author_email

	def getCreationDate(self):
		if hasattr(self, 'creation_date') and self.creation_date is not None:
			return self.creation_date.strftime('%m/%d/%Y')
		else:
			return ''

	def getCreationDateTime(self):
		if hasattr(self, 'creation_date') and self.creation_date is not None:
			return self.creation_date.strftime('%m/%d/%Y %H:%M:%S')
		else:
			return ''			

	def getNeoBoardArticleURL(self):
		url = self.absolute_url()
		return url.replace(self.getNeoBoardURL() + '/', '')			

	def getDepth(self):
		depth = count(self.getNeoBoardArticleURL(), '/')
		if depth > self._max_thread_depth:
			return self._max_thread_depth
		else:
			return depth
	
	def getParent(self):
		if self.isThreadParent():
			return None
			
		if hasattr(self, 'aq_parent'):
			return self.aq_parent
		else:
			return None

	def getThreadParent(self):
		if self.isThreadParent():
			return self

		parents = self.REQUEST.PARENTS
		for i in xrange(len(parents)-1):
			parent = parents[i]
        		if parent.isThreadParent():
        			return parent
		return None
		
	def getArticleNum(self):
		if self.isThreadParent():
			return self.id.replace('a_', '')
		else:
			return '-'
	
	def getAttachmentTypeIcon(self):
		if not self.attachment_file_id:
			return '-'
		
		mime_type, encoding = mimetypes.guess_type(self._getOb(self.attachment_file_id).getFileName())

		if not mime_type:
			return self.getIcon('mime_unknown')
		
        	type, subtype  = mime_type.split('/', 2)
        
	        if type == 'image':
			return self.getIcon('mime_img')	            
	        elif type == 'text':
	        	if subtype == 'html':
	            		return self.getIcon('mime_html')
	        	elif subtype == 'richtext':
	        		return self.getIcon('mime_docs')
	            	else:
	            		return self.getIcon('mime_text')
	        elif type == 'audio':
	        	return self.getIcon('mime_audio')	            
	        elif type == 'video':
	        	return self.getIcon('mime_video')	            	        
	        elif type == 'application':
	        	if subtype == 'zip' or subtype == 'x-tar' or subtype == 'x-gtar' or subtype == 'x-gzip':
	        		return self.getIcon('mime_zip')
	        	elif subtype == 'msword' or subtype == 'rtf':
	        		return self.getIcon('mime_docs')
	        	else:
	        		return self.getIcon('mime_exe')	        	
	        else:
			return self.getIcon('mime_unknown')
	
	def hasChildren(self):
		if len(self.objectValues('NeoBoard Article')) > 0:
			return 1
		else:
			return 0
	
	def isThreadParent(self):
		if self.aq_parent.meta_type == 'NeoBoard':
			return 1
		else:
			return 0

	def isOwner(self):
		if self.REQUEST.AUTHENTICATED_USER.has_role('Owner', self) or self.REQUEST.AUTHENTICATED_USER.has_role('Manager', self):
			return 1
		else:
			return 0

	security.declarePrivate('_getPassword')			
	def _getPassword(self):
		if hasattr(self, 'password'):
			return self.password
		else:
			return None
	
	security.declarePrivate('_getPageQueryString')			
	def _getPageQueryString(self, REQUEST):
	
		if REQUEST is None:
			return ''
			
		queries = []

		if REQUEST.has_key('expand_all'):
			queries.append('expand_all=' + REQUEST['expand_all'])		
		
		if REQUEST.has_key('start'):
			queries.append('start=' + REQUEST['start'])

		if REQUEST.has_key('skey'):
			queries.append('skey=' + REQUEST['skey'])			

		if REQUEST.has_key('reverse'):
			queries.append('reverse=' + REQUEST['reverse'])

		query_string = ''
				
		if len(queries) > 0:
			query_string = join(queries, '&')
			
			query_string = '?' + query_string
		
		return query_string

	delete = getDTML('dtml/NeoBoardArticleDelete')

	security.declareProtected('View', 'manage_deleteNeoBoardArticle')		
	def manage_deleteNeoBoardArticle(self, 
			password=None,
			REQUEST=None):
		""" Delete NP NeoBoard Article"""		                            

		if self.hasChildren():
			return MessageDialog(title='Undeletable',
	                             message="Articles with replies cannot be deleted!",
	                             action=self.absolute_url() + self._getPageQueryString(REQUEST)
	                )

		if not password == self._getPassword():
			if not self.isOwner() and not self.isNeoBoardManager():
				return MessageDialog(title='Undeletable',
		                             message="Password mismatch!",
		                             action=self.absolute_url() + self._getPageQueryString(REQUEST)
		                )		

	
		ret_url = self.aq_inner.aq_parent.absolute_url() + self._getPageQueryString(REQUEST)
		
		self.aq_inner.aq_parent._delObject(self.id)
		self.getCurrentNeoBoard()._resetNeoBoard()

		if REQUEST is not None:
            		return REQUEST.RESPONSE.redirect(ret_url)

	security.declareProtected('View', 'manage_modifyNeoBoardArticle')		
	def manage_modifyNeoBoardArticle(self, 
			title='', 
			data='',
			password=None,
			category='',
			attachment='',
			attachment_title='',
			attachment_description='',
			mail_reply=0,
			REQUEST=None):
		""" Modify NP NeoBoard Article"""		                            

		if not password == self._getPassword():				
			if not self.isOwner():		
				REQUEST.set('modify_neoboard_article', 1)
				return self.modify(REQUEST=setFormError(REQUEST, 'password', "Password mismatch!"))

		if not title:
			REQUEST.set('modify_neoboard_article', 1)
			return self.modify(REQUEST=setFormError(REQUEST, 'title', 'Please enter a title!'))

		if not data:
			REQUEST.set('modify_neoboard_article', 1)
			return self.modify(REQUEST=setFormError(REQUEST, 'data', 'Empty articles are not allowed!'))			

				
		self.title=self.menu_title=str(title)
		self.munge(data)
		self.mail_reply=mail_reply
		
		if self.isThreadParent():
			self.category=category
		else:
			self.category=''
		
		if attachment.filename:
			if not self.checkSize(attachment):
				return self.modify(REQUEST=setFormError(REQUEST, 'attachment', 'Maximum upload file size exceeded!'))		

		
			self.uploadFile(attachment, attachment_title, attachment_description, 1)

		if not self.isNeoBoardManager():
			self._setRemoteIP(REQUEST)

		self.updateContent()
		
		if REQUEST is not None:
            		return REQUEST.RESPONSE.redirect(self.absolute_url() + self._getPageQueryString(REQUEST))		

	security.declareProtected('Edit NeoBoard', 'manage_editNeoBoardArticle')
	def manage_editNeoBoardArticle(self, 
			title='', 
			data='',
			category='',
			read_count=0,
			author=None,
			creation_date=None,
			mail_reply=0,
			total_num_votes=0,
			attachment='',
			attachment_title='',
			attachment_description='',
			REQUEST=None):
		""" Edit NeoBoard Article"""		                            

		if not title:
			REQUEST.set('edit_neoboard_article', self.getNeoBoardArticleURL())
			return self.manage_editForm(REQUEST=setFormError(REQUEST, 'title', 'Please enter a title!'))

		if not data:
			REQUEST.set('edit_neoboard_article', self.getNeoBoardArticleURL())
			return self.manage_editForm(REQUEST=setFormError(REQUEST, 'data', 'Empty articles are not allowed!'))			

		
		self.title=str(title)
		
		self.read_count=read_count
		self.munge(data)
		self.mail_reply=mail_reply
		
		if self.isThreadParent():
			self.category=category
		else:
			self.category=None
		
		self.setTotalNumVotes(total_num_votes)
		if attachment.filename:
			if not self.checkSize(attachment):
				return self.manage_editForm(REQUEST=setFormError(REQUEST, 'attachment', 'Maximum upload file size exceeded!'))		
			self.uploadFile(attachment, attachment_title, attachment_description, 1)
					
		if creation_date is not None:			
			self.creation_date = DateTime(creation_date)
		
		self.updateContent()
		
		if REQUEST is not None:
			return self.manage_editForm(self, REQUEST)

	security.declareProtected('View', 'manage_addNeoBoardArticleReply')	
	def manage_addNeoBoardArticleReply(self, 
			author='',
			author_email='',
			title='', 
			data='',
			password=None,
			attachment='',
			attachment_title='',
			attachment_description='',
			mail_reply=0,			
			REQUEST=None):
		""" Add NP NeoBoard Article Reply"""		                            

		if self.isReadOnly():
			if not self.isNeoBoardManager():
				return self.addNeoBoardArticleForm(REQUEST=setFormError(REQUEST, 'title', "Read-only board. You can't post an article."))		

		if not author:
			return self.reply(setFormError(REQUEST, 'author', 'Please enter your username!'))
	
		if author_email and not isEmailValid(author_email):
			return self.reply(setFormError(REQUEST, 'author_email', 'Your email address is invalid!'))

		if not title:
			return self.reply(REQUEST=setFormError(REQUEST, 'title', 'Please enter a title!'))

		if not data:
			return self.reply(REQUEST=setFormError(REQUEST, 'data', 'Empty articles are not allowed!'))			


		instance = NeoBoardArticle()
		instance.title=str(title)
		
		instance.munge(data)
		id = 'a_' + str(self.getNextReplyNum())
		instance.id = id
		instance.category=None
		instance.setReadCount(0)
		instance.setNextReplyNum(0)
		instance.setTotalNumReplies(0)
		instance.setTotalNumVotes(0)
		instance.password = password

	
		if REQUEST is not None:
			instance.author=REQUEST.AUTHENTICATED_USER.getUserName()
		else:
			instance.author='Unknown'
		instance.creation_date = DateTime()	

		if attachment.filename:
			if not self.checkSize(attachment):
				return self.reply(REQUEST=setFormError(REQUEST, 'attachment', 'Maximum upload file size exceeded!'))		
		
		self._setObject(id, instance)

		if attachment.filename:
			self._getOb(id).uploadFile(attachment, attachment_title, attachment_description)	
		
		self.setNextReplyNum(self.getNextReplyNum())
		self.setTotalNumReplies(self.getTotalNumReplies()+1)		

		if self.isMailEnabled():
			self._getOb(id).mailArticle()
			
		if self.mail_reply:
			self.mailArticle(reply=1, reply_instance=self._getOb(id))
			
		self._getOb(id)._setRemoteIP(REQUEST)
		
		if REQUEST is not None:
            		return REQUEST.RESPONSE.redirect(self.absolute_url() + self._getPageQueryString(REQUEST))

	def manage_afterAdd(self, item, container):
	        if item is self:
			if self.hasAttachment():
				ext_file = self._getOb(self.attachment_file_id)
				org_ext_file_path = ext_file.getFilePath()
				new_ext_file_path = self.getArchivePath() + self.cookFilePath(self.attachment_file_id) +  os.path.splitext(org_ext_file_path)[1]
				
				# Must have been copied or moved
				if not os.path.isfile(new_ext_file_path):
					copyFile(org_ext_file_path, new_ext_file_path)
					os.unlink(org_ext_file_path)
					ext_file.file_path = new_ext_file_path
					

	def getSlicedTitle(self):
		depth = self.getDepth()
		
		chunk = self.getMaxListTitleLength() - (self.getDepth() * 2)
		
		return cutString(self.title, chunk)	
								
	def getNextReplyNum(self):
		return self.next_reply_num + 1

	def setNextReplyNum(self, num):
		self.next_reply_num = num
	
	def setTotalNumReplies(self, num):
		self.total_num_replies = num 

	def getTotalNumReplies(self):
		return self.total_num_replies
		
	def setTotalNumVotes(self, num):
		self.total_num_votes = num 

	def getTotalNumVotes(self):
		return self.total_num_votes
		
	def mailArticle(self, reply=0, reply_instance=None):
		
		# Enabled only on a unix platform
		if not os.environ.has_key('OSTYPE') or not os.environ['OSTYPE'] == 'linux-gnu':
			return
		
		if reply and reply_instance is not None:
			subject = '[Your Article Replied] ' + reply_instance.title
			
			template = getDTML('dtml/NeoBoardArticleReplyMail')
			body = template.__of__(reply_instance)()
			
			author_email = self.getAuthorEmail()
			reply_email = reply_instance.getAuthorEmail()
							
			self.sendmail(from_addr='[NeoBoard Mailer]', to_addr=author_email, reply_addr=reply_email, subject=subject, body=body, html=1)
		
		else:
			subject = '[NP NeoBoard Article] ' + self.title
			
			template = getDTML('dtml/NeoBoardArticleMail')
			body = template.__of__(self)()
			
			manager_email = []
			for email in self.getCurrentNeoBoard().managers.values():
				manager_email.append(email)
							
			self.sendmail(from_addr='[NeoBoard Mailer]', to_addr=join(manager_email, ','), reply_addr='[NeoBoard Mailer]', subject=subject, body=body, html=1)

	def getNextFileID(self):
		return getNextElementName(self.objectValues('NeoBoard External File'), 'FILE_')

	def cookFilePath(self, file_name):
		
		name = self.getNeoBoardArticleURL().replace('/', '_')

		return name + '__' + file_name
	
	def uploadFile(self, file, title, description, edit=0):	
		desc_file_name = os.path.basename(file.filename)
				
		if not title:
			title = desc_file_name
			
		if not description:
			description = desc_file_name	

		archive_file_name = os.path.basename(file.filename.replace(' ', '_'))				
				
		if edit:
			if not self.attachment_file_id:
				self.attachment_file_id = self.getNextFileID()				
			try:
				self._getOb(self.attachment_file_id).manage_deleteExternalFile()
			except:
				pass
			
			archive_file_path = self.getArchivePath() + self.cookFilePath(self.attachment_file_id) +  os.path.splitext(archive_file_name)[1]
			
			self.manage_addNeoBoardExternalFile(id=self.attachment_file_id, title=title, description=description, file_path=archive_file_path, hard_link=1, ext_file=file)
		else:	
			self.attachment_file_id = self.getNextFileID()				
			
			archive_file_path = self.getArchivePath() + self.cookFilePath(self.attachment_file_id) +  os.path.splitext(archive_file_name)[1]
			
			self.manage_addNeoBoardExternalFile(id=self.attachment_file_id, title=title, description=description, file_path=archive_file_path, hard_link=1, ext_file=file)
	
	def hasAttachment(self):
		if hasattr(self, 'attachment_file_id'):
			try:
				self._getOb(self.attachment_file_id)
				return 1
			except:
				return 0
		else:
			return 0
			
	def getAttachmentFile(self):
		try:
			file = self._getOb(self.attachment_file_id)
		except:
			file = None
		
		return file
		
	def getAttachmentFileURL(self):
		return self.absolute_url() + '/' + self.attachment_file_id
	
	def getCategory(self):
		if hasattr(self, 'category'):
			
			if self.category not in self.posting_categories.keys():
				self.category = ''
		
			if self.category:
				return self.category
			else:
				return None
		else:
			return None

	def getCategoryName(self):
		cat = self.getCategory()
		
		if cat:
			try:
				return self.posting_categories[cat]
			except:
				return ''
		else:
			return ''
	
	def _setRemoteIP(self, REQUEST=None):

		if REQUEST.has_key('REMOTE_HOST') and REQUEST['REMOTE_HOST']:
			self.remote_ip = REQUEST['REMOTE_HOST']
		elif self.REQUEST.has_key('REMOTE_ADDR') and REQUEST['REMOTE_ADDR']:
			self.remote_ip = REQUEST['REMOTE_ADDR']
		else:
			self.remote_ip = None
		
	def getRemoteIP(self):
		if self.remote_ip:
			return self.remote_ip
		else:
			return 'Unknown'

				
	security.declareProtected('Edit NeoBoard', 'selfUpgrade')
	def selfUpgrade(self):
		pass


class NeoBoardNotice(NeoBoardArticle):
	meta_type='NeoBoard Notice'	
	
InitializeClass(NeoBoardArticle)
InitializeClass(NeoBoardNotice)

